﻿using System;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;
using System.Linq;

namespace PakExtract
{
    class Program
    {
        static void Main(string[] args)
        {
            var pak = PakFile.FromFile("EOBDATA2.PAK");

            foreach(var file in pak.Files)
                Console.WriteLine($"{file.FileName,-12}\toffset: {file.OffsetInPak}\tsize: {file.FileSize} bytes");
            Console.WriteLine($"{pak.Files.Count} files\n");

            File.WriteAllBytes(pak.Files[0].FileName, pak.Files[0].RawData);
            var cps = CpsFile.FromData(pak.Files[0].RawData);
            SaveAsPng("test.png", cps.RawData);
            File.WriteAllBytes("test.raw", cps.RawData);

            Console.WriteLine("Filesize: \t\t" + cps.FileSize);
            Console.WriteLine("CompressionType: \t" + cps.CompressionType);
            Console.WriteLine("UncompressedSize: \t" + cps.UncompressedSize);
            Console.WriteLine("PaletteSize: \t\t" + cps.PaletteSize);
            Console.WriteLine("IsRelative: \t\t" + cps.IsRelative);

            Console.ReadKey();
        }

        public static void SaveAsPng(string filename, byte[] rawData)
        {
            Bitmap bmp = new Bitmap(320, 200);
            var palette = new Color[rawData.Max() + 1];
            Random rnd = new Random();

            for (int i = 0; i < palette.Length; i++)
                palette[i] = Color.FromArgb(rnd.Next(256), rnd.Next(256), rnd.Next(256));

            for (int i = 0; i < rawData.Length; i++)
                bmp.SetPixel(i % 320, i / 320, palette[rawData[i]]);

            bmp.Save(filename, ImageFormat.Png);
        }
    }
}
